library(rmutil)
options(contrasts=c("contr.mean","contr.poly"))

data <- read.table("ch3e2.dat",skip=6,header=T)
attach(data)

Sex <- relevel(Sex,ref="M")
print(z0 <- glm(cbind(Agree,Disagree)~1,family=binomial))
print(z1 <- glm(cbind(Agree,Disagree)~Sex,family=binomial))

EducationF <- as.factor(Education)
print(z2 <- glm(cbind(Agree,Disagree)~EducationF,family=binomial))

EducationL <- Education-mean(Education)
EducationL <- EducationL/sqrt(sum(EducationL^2))
print(z3 <- glm(cbind(Agree,Disagree)~EducationL,family=binomial))
print(z4 <- glm(cbind(Agree,Disagree)~EducationL*Sex,family=binomial))

data.frame(Agree,Disagree,Education,Sex,residuals=residuals(z4,type="pearson"))

if(interactive()){
cd <- rep(0,42)
cd[Agree+Disagree>0] <- cooks.distance(z4,res=residuals(z4,
	type="pearson")[Agree+Disagree>0],sd=1)
plot(cd,main="Cook's distance - Binomial Residuals",
	xlab="Observation number",ylab="Cook's distance",pch=8)
inf <- rep(0,42)
inf[Agree+Disagree>0] <- sqrt(1-lm.influence(z4)$hat)
qqnorm(residuals(z4,type="pearson")/inf,
	main="Q-Q Plot - Binomial Residuals",xlab="Ordered normal",
	ylab="Ordered residual",pch=8)
abline(0,1)
}

print(z5 <- glm(cbind(Agree,Disagree)~(EducationL+I(EducationL^2))*Sex,
	family=binomial))

wt <- Education>=6
print(z6 <- glm(cbind(Agree,Disagree)~EducationL*Sex,family=binomial,
	weights=wt))

if(interactive()){
plot(0:20,(Agree/(Agree+Disagree))[Sex=="M"],pch=2,xlab="Education level",
	ylab="Proportion of yes")
lines(0:20,fitted(z6)[Sex=="M"])
points(0:20,(Agree/(Agree+Disagree))[Sex=="F"],pch=8)
lines(0:20,fitted(z6)[Sex=="F"],lty=3)
}
