lmov <- function(freq,outcome,index,nord=1){
	ncols <- length(unique(outcome))
	nrows <- length(unique(index))
	lincl <- as.numeric(outcome)
	lincol <- lincl-ncols/2
	linrw <- as.numeric(index)
	linrow <- linrw-nrows/2
	scale2 <- linrow
	z <- glm(freq~scale2+outcome+scale2:outcome,family=poisson)
	dev <- z$dev
	repeat{
		scale1 <- (lincl!=1)*z$coef[lincl+ncols]+z$coef[2]
		scale1 <- (scale1-scale1[1])/(scale1[length(scale1)]-scale1[1])
		z <- glm(freq~scale1+index:scale1+index,family=poisson)
		scale2 <- (linrw!=1)*z$coef[linrw+nrows]+z$coef[2]
		scale2 <- (scale2-scale2[1])/(scale2[length(scale2)]-scale2[1])
		if(((dev-z$dev)/z$dev)^2<0.00001)break
		dev <- z$dev
		z <- glm(freq~scale2+outcome+scale2:outcome,family=poisson)}
	if(nord==1)z <- glm(freq~index+outcome+index:scale1,family=poisson)
	else z <- glm(freq~index+outcome+scale1:scale2,family=poisson)
	names(scale1) <- NULL
	names(scale2) <- NULL
	z$scale <- list(scale1,scale2)
	names(z$scale) <- c(paste(deparse(substitute(outcome))),
		paste(deparse(substitute(index))))
	z$ordlength <- c(ncols,nrows)
	z$nord <- nord
	z$df.residual <- z$df.res-nord
	class(z) <- c("lmov",class(z))
	z
}

summary.lmov <- function(z,...){
	cat("\nLog multiplicative model\n\n")
	if(z$nord==1){
		cat("Scale for",names(z$scale)[1],"\n")
		cat(unique(z$scale[[1]]),"\n\n")}
	else {
		cat("Scales:\n")
		cat(names(z$scale)[1],unique(z$scale[[1]]),"\n")
		cat(names(z$scale)[2],unique(z$scale[[2]]),"\n\n")}
	NextMethod("summary",z,...)}

print.lmov <- function(z,...){
	cat("\nLog multiplicative model\n\n")
	if(z$nord==1){
		cat("Scale for",names(z$scale)[1],"\n")
		cat(unique(z$scale[[1]]),"\n\n")}
	else {
		cat("Scales:\n")
		cat(names(z$scale)[1],unique(z$scale[[1]]),"\n")
		cat(names(z$scale)[2],unique(z$scale[[2]]),"\n\n")}
#	NextMethod("print",z,...)
	print.glm(z,...)
}

crov <- function(freq,outcome,index){
	ncols <- length(unique(outcome))
	nrows <- length(unique(index))
#	lincl <- as.numeric(outcome)
	lincl <- rep(1:length(unique(outcome)),length(unique(index)))
	nlen <- (ncols-1)*nrows
	block <- gl(ncols-1,nrows,nlen)
	index <- rep(unique(index),ncols-1)
	y <- matrix(0,ncol=2,nrow=nlen)
	y[1:nrows,1] <- freq[lincl==1]
	y[1:nrows,2] <- freq[lincl==2]
	for(i in 1:(ncols-2)){
		y[i*nrows+(1:nrows),1] <- freq[lincl==i+1]+
			y[(i-1)*nrows+(1:nrows),1]
		y[i*nrows+(1:nrows),2] <- freq[lincl==i+2]}
	z <- glm(y~block+index,family=binomial)
	z$variables <- list(y=y,block=block,index=index)
	z}

poov <- function(freq,outcome,index, print.level=0, typsiz=abs(p),
	ndigit=10, gradtol=0.00001, stepmax=10*sqrt(p%*%p), steptol=0.00001,
	iterlim=100, fscale=1){
lf <- function(p){
	g <- c(1/(1+exp(-x%*%p)),rep(1,nrows))
	g <- g[1:nlen]/g[(nrows+1):(nlen+nrows)]
	-sum(y[,1]*log(g)+y[,2]*log(1-g))
}
	call <- sys.call()
	ncols <- length(unique(outcome))
	nrows <- length(unique(index))
#	lincl <- as.numeric(outcome)
	lincl <- rep(1:length(unique(outcome)),length(unique(index)))
	nlen <- (ncols-1)*nrows
	z <- crov(freq,outcome,index)
	p <- z$coef
	y <- z$variables$y
	index <- z$variables$index
	block <- z$variables$block
	mt <- terms(~block+index)
	mf <- model.frame(mt,na.action=na.fail)
	x <- model.matrix(mt, mf)
	z0 <- nlm(lf, p=p, hessian=T, print.level=print.level, typsiz=typsiz,
		ndigit=ndigit, gradtol=gradtol, stepmax=stepmax,
		steptol=steptol, iterlim=iterlim, fscale=fscale)
	names(z0$estimate) <- dimnames(x)[[2]]
	cov <- solve(z0$hessian)
	se <- sqrt(diag(cov))
	z <- list(y=y,
		block=block,
		index=index,
		call=call,
		maxlike=z0$minimum,
		deviance=2*(z0$minimum+sum(y[,1]*log(y[,1]/(y[,1]+y[,2]))+y[,2]*log(y[,2]/(y[,1]+y[,2])))),
		df=nrows*(ncols-1)-length(z0$estimate),
		aic=z0$minimum+length(z0$estimate),
		coefficients=z0$estimate,
		se=se,
		cov=cov,
		corr=cov/(se%o%se),
		gradient=z0$gradient,
		code=z0$code)
	class(z) <- "poov"
	return(z)}

print.poov <- function(z){
	np <- length(z$coefficients)
	cat("\nCall:\n",deparse(z$call),"\n\n",sep="")
	if(z$code>2)cat("Warning: no convergence - error",z$code,"\n\n")
	cat("-Log likelihood   ",z$maxlike,"\n")
	cat("Degrees of freedom",z$df,"\n")
	cat("AIC               ",z$aic,"\n")
	cat("Deviance          ",z$dev,"\n\n")
	cat("Coefficients:\n")
	coef.table <- cbind(z$coefficients, z$se)
	nc <- max(as.numeric(z$block))
	dimnames(coef.table) <- list(names(z$coef), c("estimate", "se"))
	print.default(coef.table, digits=4, print.gap=2)
	if(np>1){
		cat("\nCorrelations:\n")
		dimnames(z$corr) <- list(seq(1,np),seq(1,np))
		print.default(z$corr, digits=4)}
	invisible(z)}
